package fullstack

import (
	"testing"

	_ "github.com/mattn/go-sqlite3"

	"github.com/stretchr/testify/assert"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/testutils"
)

func TestRegistration(t *testing.T) {
	ft := InitFullstack(t, true)
	defer ft.Shutdown()

	t.Run("Invalid account name", func(t *testing.T) {
		options := xbus.COptions{
			Name:        "valide",
			ConnOptions: ft.Server.NATSConnectOptions(),
		}
		logger := testutils.GetTestLogger(t)
		client := xbus.NewClient(options, logger)

		client.Connect()

		var req api.RegistrationRequest
		_, err := client.API.SvcRegistration.Register(req)
		assert.Equal(t, "CLIENT error: Invalid account name: ''", err.Error())

		req.Name = "nom invalide"
		_, err = client.API.SvcRegistration.Register(req)
		assert.Equal(t, "CLIENT error: Invalid account name: 'nom invalide'", err.Error())

		req.Name = "valide"
		_, err = client.API.SvcRegistration.Register(req)
		assert.Nil(t, err)
	})
}
