package demo

import (
	"sync"

	"bitbucket.org/orus-io/go-xbus"
)

// NewPrintToConsoleConsumer is a PrintToConsoleConsumer factory
func NewPrintToConsoleConsumer(*xbus.Actor) xbus.ActorService {
	return &PrintToConsoleConsumer{}
}

// PrintToConsoleConsumer is a consumer that prints the received items on
// the console
type PrintToConsoleConsumer struct{}

// Process handles an incoming process
func (c *PrintToConsoleConsumer) Process(apc *xbus.ActorProcessingContext) error {
	var wg sync.WaitGroup
	for _, input := range apc.Request.Inputs {
		er, err := apc.ReadEnvelope(input.Name)
		if err != nil {
			return err
		}
		for _, eventType := range er.EventTypes() {
			wg.Add(1)
			ch := er.GetItemChan(eventType)
			go func(eventType string) {
				for item := range ch {
					apc.LocalLog.Noticef(
						"print-to-console - %s: %s", eventType, string(item))
				}
				wg.Done()
			}(eventType)
		}
	}
	wg.Wait()
	return nil
}

func init() {
	xbus.RegisterActorService("demo.print-to-console", NewPrintToConsoleConsumer)
}
