package xbus

import (
	"errors"

	api "bitbucket.org/orus-io/go-xbus/api"
)

// SendEnvelope sends an envelope on a Actor output
func (c *Client) SendEnvelope(
	actorID api.UUID, ctx *api.ProcessingContext,
	output string, env api.Envelope,
) (status api.EnvelopeAck_ReceptionStatus, err error) {
	broker := c.API.NewBroker(actorID.String())
	ack, err := broker.Output(api.OutputRequest{
		Context:  ctx,
		Output:   output,
		Envelope: &env,
	})
	if err != nil {
		c.Log.Errorf("Error sending envelope: %s", err)
		return
	}
	status = ack.GetStatus()
	if ack.GetStatus() == api.EnvelopeAck_ERROR {
		err = errors.New(ack.GetReason())
		c.Log.Errorf("Error sending envelope: %s", err)
		return
	}
	return
}
